#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Auteur :  Jonas Fourquier (http://jonas.tuxfamily.org
# Licence : GNU GPL

if __name__ == "__main__":
    import notecalc

import os,gtk,gzip
import ncalc_cst

#debugging
import pdb, time
#pdb.set_trace()

class save:
    "Sauvegarde du fichier"

    def __init__(self, info, calc, file=None, speedSave=False, saveAs = False):
        '''Initialisation sauvegarde, si le nom du fichier n'est pas
        spécifié, affiche une boite de dialogue.
        speedSave est une sauvegarde rapide (nom compressée) dans le dossier
        temporaire $HOME/.notecalc'''

        print '*** Sauvegarder %s ***'%file

        #~ pdb.set_trace()

        os.chdir(ncalc_cst.get_currentdir())
        self.xmlfile = 'notecalc.xml' #achanger aussi dans la classe load
        self.picturesdir = 'pictures'

        if not file:
            saveAs = True

        if not saveAs or speedSave:
            '''On ne demande pas un "SaveAs" et le nom du fichier est connu'''
            self.write (info, calc, file, speedSave)
        else:
            wParent = None
            chooser = gtk.FileChooserDialog("Enregister sous ...",
                                            wParent,
                                            gtk.FILE_CHOOSER_ACTION_SAVE,
                                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                            gtk.STOCK_SAVE, gtk.RESPONSE_OK));

            if file:
                chooser.set_current_folder(file)
            elif 'HOME' in os.environ :
                chooser.set_current_folder(os.path.expanduser('~'))

            filtre = gtk.FileFilter()
            filtre.set_name("notecalc")
            filtre.add_pattern("*.notecalc")
            chooser.add_filter(filtre)

            filtre = gtk.FileFilter()
            filtre.set_name("All files")
            filtre.add_pattern("*")
            chooser.add_filter(filtre)

            if (chooser.run() == gtk.RESPONSE_OK):
                self.write (info, calc, chooser.get_filename(), speedSave)

            chooser.destroy()


    def write (self, info, calcs, file, speedSave=False):
        "Ecriture du fichier xml"
        from xml.dom.minidom import Document

        doc = Document()

        racine = doc.createElement("notecalc")
        doc.appendChild(racine)

        #Entête du fichier
        infoXml = doc.createElement("info")

        #pdb.set_trace()

        print info
        infoXml.setAttribute('title',info['title'])
        infoXml.setAttribute('author',info['author'])
        infoXml.setAttribute('date',str(int(time.mktime(info['date']))))
        desc = doc.createTextNode(info['description'])
        infoXml.appendChild(desc)
        #~ infoXml.appendChild(doc.createElement('description').appendChild(doc.createTextNode(info['description'])))
        racine.appendChild(infoXml)
        print infoXml.toprettyxml()


        #content
        content = doc.createElement("content")

        #line
        for calc in calcs:
            '''stock chaque ligne de calcul'''

            #parent
            line = doc.createElement("line")
            line.setAttribute('id',   str(calc[0]))     #id de la ligne
            line.setAttribute('type', str(calc[11]))    #type d'entrée
            line.setAttribute('name', str(calc[1]))     #nom de la ligne (vide si aucun)

            #entréer utilisateur
            entry = doc.createElement('entry')
            entry.appendChild(doc.createTextNode(calc[2]))
            line.appendChild(entry)
            print entry.toprettyxml()

            #paramêtre de rendu (varie si c'est une image)
            render = doc.createElement("render")
            if calc[11] == 'img':
                '''type image'''
                render.setAttribute('name',         calc[4]['name'])
                render.setAttribute('file_name',    calc[4]['file_name'])
                render.setAttribute('original_path',calc[4]['original_path'])
                render.setAttribute('size',         str(calc[4]['print_size']))
            else :
                '''autre type'''
                (algebre,numeric,latex) = calc[4]
                if algebre: render.setAttribute('algebre','1')
                else :      render.setAttribute('algebre','0')
                if numeric: render.setAttribute('numeric','1')
                else :      render.setAttribute('numeric','0')
                if latex:   render.setAttribute('latex'  ,'1')
                else :      render.setAttribute('latex'  ,'0')
                render.setAttribute('precision',str(calc[5]))
            line.appendChild(render)
            print render.toprettyxml()

            #liste des id ou chercher les variable
            vars = doc.createElement("list_vars")
            for (name,lineid) in calc[9].items() :
                '''Stock l'id de la ligne au chercher la variable'''
                var = doc.createElement("var")
                var.setAttribute('name',str(name))
                var.setAttribute('lineid',str(lineid))
                vars.appendChild(var)
                print var.toprettyxml()
            line.appendChild(vars)
            print vars.toprettyxml()

            print line.toprettyxml()

            content.appendChild(line)
            print content.toprettyxml()

            # → ligne suivante

        racine.appendChild(content)

        print "*** Fichier XML créer ***\n\nstart xmlFile ************\
            \n%s\n\
            ********** end xmlFile" %doc.toprettyxml()

        #Enregistrement
        #pdb.set_trace()

        f = open(self.xmlfile,"w")
        f.write(doc.toprettyxml())
        f.close()

        #ff = open('current',"r")
        #print '***Contenu du fichier****\n%s\n******endfile*****'%ff.read()
        #ff.close
        if not speedSave:
            '''Sauvegarde des images et du xml dans une archive Zip'''
            import zipfile

            if not file[-9:].lower() == '.notecalc':
                file = '.'.join((file,'notecalc'))

            fz = zipfile.ZipFile(file,'w')
            fz.write(self.xmlfile)

            for file in os.listdir(self.picturesdir):
                file = os.path.join(self.picturesdir,file)
                if os.path.isfile(file): fz.write(file)

            fz.close()

        #pdb.set_trace()
        return




class load:
    def __init__(self):
        self.xmlfile = 'notecalc.xml'

    def start(self,file=None):
        print '*** Chargement %s ***'%file
        if file:
            self.read(file)
        else:
            chooser = gtk.FileChooserDialog("Ouvrir ...",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK));

            try :
                chooser.set_current_folder(os.environ['HOME'])
            except :
                pass

            filtre = gtk.FileFilter()
            filtre.set_name("notecalc")
            filtre.add_pattern("*.notecalc")
            chooser.add_filter(filtre)

            filtre = gtk.FileFilter()
            filtre.set_name("All files")
            filtre.add_pattern("*")
            chooser.add_filter(filtre)

            if (chooser.run() == gtk.RESPONSE_OK):
                print '*** ouverture de %s ***'%chooser.get_filename()
                result =  self.read(chooser.get_filename())
            else :
                result = (False,False,False)

            chooser.destroy()
            return result


    def read(self,filePath):
        #Dezipage
        import zipfile
        from xml.dom.minidom import parse
        import image
        wimg = image.image

        try :
            zfile = zipfile.ZipFile(filePath,'r')
        except :
            errordialogue = gtk.MessageDialog(
                        None, #parent
                        gtk.DIALOG_DESTROY_WITH_PARENT, #flag
                        gtk.MESSAGE_WARNING, # type
                        gtk.BUTTONS_OK, # boutons
                        _("Type de fichier incorrect."))#Message
            errordialogue.run()
            errordialogue.destoy()
            return

        for filename in zfile.namelist():
            data = zfile.read(filename)
            #pdb.set_trace()
            #if filename == 'xml':
                #filexml = data
            file = open(os.path.join(os.environ['NOTECALC_USERDIR'],ncalc_cst.get_currentdir(),filename), 'w+b')
            file.write(data)
            file.close()

        if not os.path.exists(os.path.join(os.environ['NOTECALC_USERDIR'],ncalc_cst.get_currentdir(),self.xmlfile)):
            errordialogue = gtk.MessageDialog(
                        None, #parent
                        gtk.DIALOG_DESTROY_WITH_PARENT, #flag
                        gtk.MESSAGE_WARNING, # type
                        gtk.BUTTONS_OK, # boutons
                        _("Erreur aucun fichier xml n'a été trouvé, êtes vous sûr qu'il s'agisse d'un fichier notecalc ?"))#Message
            errordialogue.run()
            errordialogue.destoy()
            return

        docxml = parse(os.path.join(os.environ['NOTECALC_USERDIR'],ncalc_cst.get_currentdir(),self.xmlfile))

        #info
        infoNode = docxml.getElementsByTagName('info')[0]

        dicoInfo = {
            'title'         : infoNode.attributes['title'].value,
            'author'        : infoNode.attributes['author'].value,
            'date'          : time.localtime(float(infoNode.attributes['date'].value)),
            'description'   : infoNode.childNodes[0].toxml().strip(),
            }


        #for (key,value) in dicoInfo.items():
            #print '%s = %s\n'%(key,value)

        #calc
        contentNode = docxml.getElementsByTagName('content')[0]
        storeCalc = []
        for lineNode in contentNode.getElementsByTagName('line'):
            #~ pdb.set_trace()
            id = int(lineNode.attributes['id'].value)
            name = lineNode.attributes['name'].value
            type = lineNode.attributes['type'].value

            entry = lineNode.getElementsByTagName('entry')[0].childNodes[0].toxml().strip()

            renderNode = lineNode.getElementsByTagName('render')[0]
            if type == 'img':
                '''entrée de type image'''
                file_name = renderNode.attributes['file_name'].value
                print_size= float(renderNode.attributes['size'].value)
                aff = {
                        'name' :        renderNode.attributes['name'].value,
                        'file_name':    file_name,
                        'original_path':renderNode.attributes['original_path'].value,
                        'print_size':   print_size,
                        'pixbuf':       None#wimg.get_pbthumb(os.path.join(os.environ['NOTECALC_USERDIR'],ncalc_cst.get_currentdir(),'pictures',file_name),print_size)
                    }
                prec=0
            else:
                if renderNode.attributes['algebre'].value == "1" : algebre = True
                else:                                           algebre = False
                if renderNode.attributes['numeric'].value == "1" : numeric = True
                else:                                           numeric = False
                if renderNode.attributes['latex'].value   == "1" : latex   = True
                else:                                           latex   = False
                aff = (algebre, numeric, latex)
                prec = int(renderNode.attributes['precision'].value)

            varsNode = lineNode.getElementsByTagName('list_vars')[0]
            dicoVars = {} #dico de variable
            for varNode in varsNode.getElementsByTagName('var'):
                dicoVars[varNode.attributes['name'].value] = int(varNode.attributes['lineid'].value)

            storeCalc.append((
                        id,             # int     # 0 # id
                        '',             # str     # 1 # nom de la variable
                        entry,          # str     # 2 # entrée de l'utilisateur
                        '[loading...]', # str     # 3 # chaine rendu
                        aff,            # object  # 4 # affichage (algebre,numeric,latex)
                        prec,           # int     # 5 # précision de l'affichage du resultat
                        None,           # float   # 6 # Résultat
                        '?',            # str     # 7 # unitée
                        [],             # object  # 8 # Liste de ligne liée
                        dicoVars ,      # object  # 9 # Dico des ids des variables
                        '',             # str     # 10# commentaire
                        type            # str     # 11# type
                ))

        print "***dicoInfo***\n%s\n\n****storeCalc****\n%s"%(dicoInfo, storeCalc)


        return (filePath, dicoInfo, storeCalc)



if __name__ == '__main__':
    import notecalc #pour les variables d'environnement
    '''Pour les tester : dicos bidons'''
    from time import strftime
    info = {
        'title': 'Titre',
        'description': 'blabla',
        'author': 'JF',
        'date' : strftime('%d/%m/%Y %H:%M')
        }
    calc = [
    #     (ID,  entry, render, aff ,precRes, res,   unit, listVar, comm)
        #{'id' :'1',   'entry': '2+2', 'render':'2+2', 'aff': '2', 'precRes':'1', 'res':'5', 'unit':'kN', 'listVar':(1,2), 'comm':'blabla'}
        ('1', '2+2/*dicobidon*/', '2+2/*dicobidon*/', '2', '1', 'yen a d autre mais inutile pour test')
        ]

    #save = save(info, calc, '/home/jonas/aaa.notecalc')
    #load = load('/home/jonas/aaa.notecalc')


    gtk.main()
import os


#file = open(PathFile,"a")
#file.write(str(filexml.write(entete, listcalc, PathFile, 'false')))
#file.close
